@@ -163,7 +163,8 @@ class Etcd3Lock(locking.Lock):
         return False
 
 
-class Etcd3Driver(coordination.CoordinationDriverWithExecutor):
+class Etcd3Driver(coordination.CoordinationDriverCachedRunWatchers,
+                  coordination.CoordinationDriverWithExecutor):
     """An etcd based driver.
 
     This driver uses etcd provide the coordination driver semantics and
@@ -180,6 +181,7 @@ class Etcd3Driver(coordination.CoordinationDriverWithExecutor):
     ==================  =======
     Name                Default
     ==================  =======
+    api_version         v3
     ca_cert             None
     cert_key            None
     cert_cert           None
@@ -198,6 +200,9 @@ class Etcd3Driver(coordination.CoordinationDriverWithExecutor):
     #: Default port used if none provided (4001 or 2379 are the common ones).
     DEFAULT_PORT = 2379
 
+    #: Default api version if none provided
+    DEFAULT_API_VERSION = "v3"
+
     GROUP_PREFIX = b"tooz/groups/"
 
     def __init__(self, member_id, parsed_url, options):
@@ -210,12 +215,14 @@ class Etcd3Driver(coordination.CoordinationDriverWithExecutor):
         cert_key = options.get('cert_key')
         cert_cert = options.get('cert_cert')
         timeout = int(options.get('timeout', self.DEFAULT_TIMEOUT))
+        api_version = options.get("api_version", self.DEFAULT_API_VERSION)
         self.client = etcd3gw.client(host=host,
                                      port=port,
                                      protocol=protocol,
                                      ca_cert=ca_cert,
                                      cert_key=cert_key,
                                      cert_cert=cert_cert,
+                                     api_path="/" + api_version + "/",
                                      timeout=timeout)
         self.lock_timeout = int(options.get('lock_timeout', timeout))
         self.membership_timeout = int(options.get(
@@ -242,18 +249,6 @@ class Etcd3Driver(coordination.CoordinationDriverWithExecutor):
             lock.heartbeat()
         return min(self.lock_timeout, self.membership_timeout)
 
-    def watch_join_group(self, group_id, callback):
-        raise tooz.NotImplemented
-
-    def unwatch_join_group(self, group_id, callback):
-        raise tooz.NotImplemented
-
-    def watch_leave_group(self, group_id, callback):
-        raise tooz.NotImplemented
-
-    def unwatch_leave_group(self, group_id, callback):
-        raise tooz.NotImplemented
-
     def _encode_group_id(self, group_id):
         return _encode(self._prefix_group(group_id))
 
@@ -443,3 +438,11 @@ class Etcd3Driver(coordination.CoordinationDriverWithExecutor):
                 group[1]['key'][len(self.GROUP_PREFIX):-1] for group in groups]
         return coordination.CoordinatorResult(
             self._executor.submit(_get_groups))
+
+    @staticmethod
+    def watch_elected_as_leader(group_id, callback):
+        raise tooz.NotImplemented
+
+    @staticmethod
+    def unwatch_elected_as_leader(group_id, callback):
+        raise tooz.NotImplemented
