@@ -14,7 +14,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import contextlib
 from distutils import version
 import functools
 import logging
@@ -26,9 +25,6 @@ from oslo_utils import strutils
 import redis
 from redis import exceptions
 from redis import sentinel
-import six
-from six.moves import map as compat_map
-from six.moves import zip as compat_zip
 
 import tooz
 from tooz import coordination
@@ -38,24 +34,51 @@ from tooz import utils
 LOG = logging.getLogger(__name__)
 
 
-@contextlib.contextmanager
-def _translate_failures():
-    """Translates common redis exceptions into tooz exceptions."""
-    try:
-        yield
-    except (exceptions.ConnectionError, exceptions.TimeoutError) as e:
-        utils.raise_with_cause(coordination.ToozConnectionError,
-                               encodeutils.exception_to_unicode(e),
-                               cause=e)
-    except exceptions.RedisError as e:
-        utils.raise_with_cause(tooz.ToozError,
-                               encodeutils.exception_to_unicode(e),
-                               cause=e)
+def _handle_failures(n_tries=15):
+
+    """Translates common redis exceptions into tooz exceptions.
+
+    This also enables retrying on certain exceptions.
+
+    :param func: the function to act on
+    :param n_tries: the number of retries
+    """
+    def inner(func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+            ntries = n_tries
+            while ntries:
+                try:
+                    return func(*args, **kwargs)
+                except exceptions.ConnectionError as e:
+                    # retry ntries times and then raise a connection error
+                    ntries -= 1
+                    if not ntries:
+                        LOG.debug(
+                            "Redis connection error, "
+                            "retry limit has been reached, aborting - %s", e
+                        )
+                        utils.raise_with_cause(
+                            coordination.ToozConnectionError,
+                            encodeutils.exception_to_unicode(e),
+                            cause=e)
+                    LOG.debug("Redis connection error, will retry - %s", e)
+
+                except (exceptions.TimeoutError) as e:
+                    utils.raise_with_cause(coordination.ToozConnectionError,
+                                           encodeutils.exception_to_unicode(e),
+                                           cause=e)
+                except exceptions.RedisError as e:
+                    utils.raise_with_cause(tooz.ToozError,
+                                           encodeutils.exception_to_unicode(e),
+                                           cause=e)
+        return wrapper
+    return inner
 
 
 class RedisLock(locking.Lock):
     def __init__(self, coord, client, name, timeout):
-        name = "%s_%s_lock" % (coord.namespace, six.text_type(name))
+        name = "%s_%s_lock" % (coord.namespace, str(name))
         super(RedisLock, self).__init__(name)
         # NOTE(jd) Make sure we don't release and heartbeat at the same time by
         # using a exclusive access lock (LP#1557593)
@@ -66,48 +89,48 @@ class RedisLock(locking.Lock):
         self._coord = coord
         self._client = client
 
+    @_handle_failures()
     def is_still_owner(self):
-        with _translate_failures():
-            lock_tok = self._lock.local.token
-            if not lock_tok:
-                return False
-            owner_tok = self._client.get(self.name)
-            return owner_tok == lock_tok
+        lock_tok = self._lock.local.token
+        if not lock_tok:
+            return False
+        owner_tok = self._client.get(self.name)
+        return owner_tok == lock_tok
 
+    @_handle_failures()
     def break_(self):
-        with _translate_failures():
-            return bool(self._client.delete(self.name))
+        return bool(self._client.delete(self.name))
 
+    @_handle_failures()
     def acquire(self, blocking=True, shared=False):
         if shared:
             raise tooz.NotImplemented
         blocking, timeout = utils.convert_blocking(blocking)
-        with _translate_failures():
-            acquired = self._lock.acquire(
-                blocking=blocking, blocking_timeout=timeout)
-            if acquired:
-                with self._exclusive_access:
-                    self._coord._acquired_locks.add(self)
-            return acquired
+        acquired = self._lock.acquire(
+            blocking=blocking, blocking_timeout=timeout)
+        if acquired:
+            with self._exclusive_access:
+                self._coord._acquired_locks.add(self)
+        return acquired
 
+    @_handle_failures()
     def release(self):
         with self._exclusive_access:
-            with _translate_failures():
-                try:
-                    self._lock.release()
-                except exceptions.LockError as e:
-                    LOG.error("Unable to release lock '%r': %s", self, e)
-                    return False
-                finally:
-                    self._coord._acquired_locks.discard(self)
-                return True
+            try:
+                self._lock.release()
+            except exceptions.LockError as e:
+                LOG.error("Unable to release lock '%r': %s", self, e)
+                return False
+            finally:
+                self._coord._acquired_locks.discard(self)
+            return True
 
+    @_handle_failures()
     def heartbeat(self):
         with self._exclusive_access:
             if self.acquired:
-                with _translate_failures():
-                    self._lock.reacquire()
-                    return True
+                self._lock.reacquire()
+                return True
         return False
 
     @property
@@ -356,7 +379,7 @@ return 1
         self._scripts = {}
 
     def _check_fetch_redis_version(self, geq_version, not_existent=True):
-        if isinstance(geq_version, six.string_types):
+        if isinstance(geq_version, str):
             desired_version = version.LooseVersion(geq_version)
         elif isinstance(geq_version, version.LooseVersion):
             desired_version = geq_version
@@ -431,11 +454,12 @@ return 1
             if 'sentinel_fallback' in kwargs:
                 del kwargs['sentinel_fallback']
             master_client = sentinel_server.master_for(sentinel_name, **kwargs)
-            # The master_client is a redis.StrictRedis using a
+            # The master_client is a redis.Redis using a
             # Sentinel managed connection pool.
             return master_client
-        return redis.StrictRedis(**kwargs)
+        return redis.Redis(**kwargs)
 
+    @_handle_failures()
     def _start(self):
         super(RedisDriver, self)._start()
         try:
@@ -449,8 +473,7 @@ return 1
             # Ensure that the server is alive and not dead, this does not
             # ensure the server will always be alive, but does insure that it
             # at least is alive once...
-            with _translate_failures():
-                self._server_info = self._client.info()
+            self._server_info = self._client.info()
             # Validate we have a good enough redis version we are connected
             # to so that the basic set of features we support will actually
             # work (instead of blowing up).
@@ -470,12 +493,12 @@ return 1
             # For py3.x ensure these are unicode since the string template
             # replacement will expect unicode (and we don't want b'' as a
             # prefix which will happen in py3.x if this is not done).
-            for (k, v) in six.iteritems(tpl_params.copy()):
-                if isinstance(v, six.binary_type):
+            for (k, v) in tpl_params.copy().items():
+                if isinstance(v, bytes):
                     v = v.decode('ascii')
                 tpl_params[k] = v
             prepared_scripts = {}
-            for name, raw_script_tpl in six.iteritems(self.SCRIPTS):
+            for name, raw_script_tpl in self.SCRIPTS.items():
                 script_tpl = string.Template(raw_script_tpl)
                 script = script_tpl.substitute(**tpl_params)
                 prepared_scripts[name] = self._client.register_script(script)
@@ -509,12 +532,13 @@ return 1
     def _decode_group_id(self, group_id):
         return utils.to_binary(group_id, encoding=self._encoding)
 
+    @_handle_failures()
     def heartbeat(self):
-        with _translate_failures():
-            beat_id = self._encode_beat_id(self._member_id)
-            expiry_ms = max(0, int(self.membership_timeout * 1000.0))
-            self._client.psetex(beat_id, time_ms=expiry_ms,
-                                value=self.STILL_ALIVE)
+        beat_id = self._encode_beat_id(self._member_id)
+        expiry_ms = max(0, int(self.membership_timeout * 1000.0))
+        self._client.psetex(beat_id, time_ms=expiry_ms,
+                            value=self.STILL_ALIVE)
+
         for lock in self._acquired_locks.copy():
             try:
                 lock.heartbeat()
@@ -523,6 +547,7 @@ return 1
                             exc_info=True)
         return min(self.lock_timeout, self.membership_timeout)
 
+    @_handle_failures()
     def _stop(self):
         while self._acquired_locks:
             lock = self._acquired_locks.pop()
@@ -537,8 +562,7 @@ return 1
             try:
                 # NOTE(harlowja): this will delete nothing if the key doesn't
                 # exist in the first place, which is fine/expected/desired...
-                with _translate_failures():
-                    self._client.delete(beat_id)
+                self._client.delete(beat_id)
             except tooz.ToozError:
                 LOG.warning("Unable to delete heartbeat key '%s'", beat_id,
                             exc_info=True)
@@ -630,10 +654,10 @@ return 1
                 return set()
             # Ok now we need to see which members have passed away...
             gone_members = set()
-            member_values = p.mget(compat_map(self._encode_beat_id,
-                                              potential_members))
-            for (potential_member, value) in compat_zip(potential_members,
-                                                        member_values):
+            member_values = p.mget(map(self._encode_beat_id,
+                                       potential_members))
+            for (potential_member, value) in zip(potential_members,
+                                                 member_values):
                 # Always preserve self (just incase we haven't heartbeated
                 # while this call/s was being made...), this does *not* prevent
                 # another client from removing this though...
@@ -742,7 +766,7 @@ return 1
         return self.get_lock(name)
 
     def run_elect_coordinator(self):
-        for group_id, hooks in six.iteritems(self._hooks_elected_leader):
+        for group_id, hooks in self._hooks_elected_leader.items():
             leader_lock = self._get_leader_lock(group_id)
             if leader_lock.acquire(blocking=False):
                 # We got the lock
@@ -755,5 +779,4 @@ return 1
         return result
 
 
-RedisFutureResult = functools.partial(coordination.CoordinatorResult,
-                                      failure_translator=_translate_failures)
+RedisFutureResult = functools.partial(coordination.CoordinatorResult)
