@@ -19,7 +19,6 @@ import os
 
 import fixtures
 from oslo_utils import uuidutils
-import six
 from testtools import testcase
 
 import tooz
@@ -49,14 +48,20 @@ class SkipNotImplementedMeta(type):
         return type.__new__(cls, name, bases, local)
 
 
-@six.add_metaclass(SkipNotImplementedMeta)
-class TestWithCoordinator(testcase.TestCase):
+class TestWithCoordinator(testcase.TestCase, metaclass=SkipNotImplementedMeta):
     url = os.getenv("TOOZ_TEST_URL")
 
     def setUp(self):
         super(TestWithCoordinator, self).setUp()
         if self.url is None:
             raise RuntimeError("No URL set for this driver")
+        if os.getenv("TOOZ_TEST_ETCD3"):
+            self.url = self.url.replace("etcd://", "etcd3://")
+        if os.getenv("TOOZ_TEST_ETCD3GW"):
+            # TODO(jan.gutter): When pifpaf supports etcd 3.4 we should use the
+            # defaults
+            self.url = self.url.replace("etcd://", "etcd3+http://") + \
+                "?api_version=v3beta"
         self.useFixture(fixtures.NestedTempfile())
         self.group_id = get_random_uuid()
         self.member_id = get_random_uuid()
